@@ -7,8 +7,8 @@ from yatl.helpers import TAG
 from py4web import action, URL, request
 from py4web.core import dumps, Session, Fixture
 
-class Inject(Fixture):
 
+class Inject(Fixture):
     def __init__(self, **variables):
         self.variables = variables
 
@@ -19,7 +19,6 @@ class Inject(Fixture):
 
 
 class ActionFactory:
-
     def __init__(self, *fixtures):
         self.fixtures = fixtures
 
@@ -38,7 +37,9 @@ class ActionFactory:
     def head(self, path=None, template=None):
         return self._action_maker("HEAD", path, template)
 
-    def __call__(self, path=None, template=None, method=['GET', 'POST', 'PUT', 'HEAD', 'DELETE']):
+    def __call__(
+        self, path=None, template=None, method=["GET", "POST", "PUT", "HEAD", "DELETE"]
+    ):
         return self._action_maker(method, path, template)
 
     def _action_maker(self, method, path, template):
