@@ -89,6 +89,7 @@ class GridClassStyle:
         "search_form_table": "search-form-table",
         "search_form_tr": "search-form-tr",
         "search_form_td": "search-form-td",
+        "search_boolean": "search-boolean",
     }
 
     styles = {
@@ -128,6 +129,7 @@ class GridClassStyle:
         "search_form_table": "",
         "search_form_tr": "",
         "search_form_td": "",
+        "search_boolean": "",
     }
 
     @classmethod
@@ -180,6 +182,7 @@ class GridClassStyleBulma(GridClassStyle):
         "search_form_table": "search-form-table",
         "search_form_tr": "search-form-tr",
         "search_form_td": "search-form-td pr-1",
+        "search_boolean": "search-boolean",
     }
 
     styles = {
@@ -219,6 +222,7 @@ class GridClassStyleBulma(GridClassStyle):
         "search_form_table": "",
         "search_form_tr": "",
         "search_form_td": "",
+        "search_boolean": "padding-top: .5rem;",
     }
 
 
@@ -249,17 +253,17 @@ class Grid:
         else "",
         "date": lambda value: XML(
             '<script>document.write((new Date(%s,%s,%s)).toLocaleString().split(",")[0])</script>'
-            % (value.year, value.month, value.day,)
+            % (
+                value.year,
+                value.month,
+                value.day,
+            )
         )
         if value
         else "",
-        "list:string": lambda value: ', '.join(str(x) for x in value)
-        if value
-        else "",
-        "list:integer": lambda value: ', '.join(x for x in value)
-        if value
-        else "",
-        "default": lambda value: str(value) if value is not None else ""
+        "list:string": lambda value: ", ".join(str(x) for x in value) if value else "",
+        "list:integer": lambda value: ", ".join(x for x in value) if value else "",
+        "default": lambda value: str(value) if value is not None else "",
     }
 
     def __init__(
@@ -334,6 +338,16 @@ class Grid:
             search_button_text=search_button_text,
             formstyle=formstyle,
             grid_class_style=grid_class_style,
+            new_sidecar=None,
+            new_submit_value=None,
+            new_action_button_text="New",
+            details_sidecar=None,
+            details_submit_value=None,
+            details_action_button_text="Details",
+            edit_sidecar=None,
+            edit_submit_value=None,
+            edit_action_button_text="Edit",
+            delete_action_button_text="Delete",
         )
 
         #  instance variables that will be computed
@@ -427,6 +441,22 @@ class Grid:
                 readonly=readonly,
                 formstyle=self.param.formstyle,
             )
+            if self.action == "new":
+                if self.param.new_sidecar:
+                    self.form.param.sidecar.append(self.param.new_sidecar)
+                if self.param.new_submit_value:
+                    self.form.param.submit_value = self.param.new_submit_value
+            if self.action == "details":
+                if self.param.details_sidecar:
+                    self.form.param.sidecar.append(self.param.details_sidecar)
+                if self.param.details_submit_value:
+                    self.form.param.submit_value = self.param.details_submit_value
+            if self.action == "edit":
+                if self.param.edit_sidecar:
+                    self.form.param.sidecar.append(self.param.edit_sidecar)
+                if self.param.edit_submit_value:
+                    self.form.param.submit_value = self.param.edit_submit_value
+
             # SECURITY: if the new record was created but does not match filter, delete it
             if self.form.accepted and not self.record_id:
                 new_record = db[self.tablename]._id == self.form.vars["id"]
@@ -454,7 +484,10 @@ class Grid:
             pt = db[self.tablename]
             key_is_missing = True
             for field in self.param.fields:
-                if field.table._tablename == pt._tablename and field.name == pt._id.name:
+                if (
+                    field.table._tablename == pt._tablename
+                    and field.name == pt._id.name
+                ):
                     key_is_missing = False
             if key_is_missing:
                 #  primary key wasn't included, add it and set show_id to False so it doesn't display
@@ -554,6 +587,7 @@ class Grid:
         override_classes=None,
         override_styles=None,
         message=None,
+        onclick=None,
         row_id=None,
         name="grid-button",
         row=None,
@@ -562,7 +596,7 @@ class Grid:
         separator = "?"
         if row_id:
             url += "/%s" % row_id
- 
+
         classes = self.param.grid_class_style.classes.get(name, "")
         styles = self.param.grid_class_style.styles.get(name, "")
 
@@ -594,7 +628,12 @@ class Grid:
             **attr,
         )
         if self.param.include_action_button_text:
-            link.append(XML('<span class="grid-action-button-text">&nbsp;%s</span>' % button_text))
+            link.append(
+                XML(
+                    '<span class="grid-action-button-text">&nbsp;%s</span>'
+                    % button_text
+                )
+            )
 
         return link
 
@@ -612,8 +651,17 @@ class Grid:
             if not key in ("search_type", "search_string")
         ]
         form = FORM(*hidden_fields, **dict(_method="GET", _action=self.endpoint))
-        select = SELECT(*options, **dict(_name="search_type",))
-        input = INPUT(_type="text", _name="search_string", _value=search_string,)
+        select = SELECT(
+            *options,
+            **dict(
+                _name="search_type",
+            ),
+        )
+        input = INPUT(
+            _type="text",
+            _name="search_string",
+            _value=search_string,
+        )
         submit = INPUT(_type="submit", _value="Search")
         clear_script = "document.querySelector('[name=search_string]').value='';"
         clear = INPUT(_type="submit", _value="Clear", _onclick=clear_script)
@@ -637,8 +685,10 @@ class Grid:
         for field in self.param.search_form.table:
             td = TD(**self.param.grid_class_style.get("search_form_td"))
             if field.type == "boolean":
-                td.append(self.param.search_form.custom["widgets"][field.name])
-                td.append(field.label)
+                sb = DIV(**self.param.grid_class_style.get("search_boolean"))
+                sb.append(self.param.search_form.custom["widgets"][field.name])
+                sb.append(field.label)
+                td.append(sb)
             else:
                 td.append(self.param.search_form.custom["widgets"][field.name])
             if (
@@ -720,7 +770,9 @@ class Grid:
             )
 
         if self.param.details or self.param.editable or self.param.deletable:
-            thead.append(TH("", **self.param.grid_class_style.get('grid-th-action-button')))
+            thead.append(
+                TH("", **self.param.grid_class_style.get("grid-th-action-button"))
+            )
 
         return thead
 
@@ -745,18 +797,22 @@ class Grid:
         formatter = (
             self.formatters.get(key)
             or self.formatters_by_type.get(field.type)
-            or self.formatters_by_type.get('default')
+            or self.formatters_by_type.get("default")
         )
 
-        class_type = "grid-cell-type-%s" % str(field.type).split(":")[0]
+        class_type = "grid-cell-type-%s" % str(field.type).split(":")[0].split("(")[0]
         class_col = "grid-col-%s" % key
         td = TD(
             formatter(field_value),
             _class=(
                 self.param.grid_class_style.classes.get("grid-td", "")
                 + " "
-                + class_type if class_type not in self.param.grid_class_style.classes.get(class_type,
-                                                                                          "").split(" ") else ""
+                + class_type
+                if class_type
+                not in self.param.grid_class_style.classes.get(class_type, "").split(
+                    " "
+                )
+                else ""
                 + " "
                 + self.param.grid_class_style.classes.get(class_type, "")
                 + " "
@@ -798,32 +854,35 @@ class Grid:
                 or (self.param.editable and self.param.editable != "")
                 or (self.param.deletable and self.param.deletable != "")
             ):
-                classes = (self.param.grid_class_style.classes.get("grid-td", "") +
-                           " " +
-                           self.param.grid_class_style.classes.get("grid-td-action-button")
-                ).strip() 
-                styles = (self.param.grid_class_style.styles.get("grid-td", "") +
-                           " " +
-                           self.param.grid_class_style.styles.get("grid-td-action-button")
+                classes = (
+                    self.param.grid_class_style.classes.get("grid-td", "")
+                    + " "
+                    + self.param.grid_class_style.classes.get("grid-td-action-button")
+                ).strip()
+                styles = (
+                    self.param.grid_class_style.styles.get("grid-td", "")
+                    + " "
+                    + self.param.grid_class_style.styles.get("grid-td-action-button")
                 ).strip()
                 td = TD(_class=classes, _style=styles)
                 if self.param.pre_action_buttons:
                     for btn in self.param.pre_action_buttons:
+                        if btn.onclick:
+                            btn.url = None
                         td.append(
                             self.render_action_button(
                                 btn.url,
                                 btn.text,
                                 btn.icon,
+                                _onclick=btn.onclick,
                                 additional_classes=btn.additional_classes,
                                 message=btn.message,
                                 row_id=row_id if btn.append_id else None,
-                                row=row
+                                row=row,
                             )
                         )
                 if self.param.details and self.param.details != "":
-                    if isinstance(
-                        self.param.details, str
-                    ):
+                    if isinstance(self.param.details, str):
                         details_url = self.param.details
                     else:
                         details_url = self.endpoint + "/details"
@@ -831,7 +890,7 @@ class Grid:
                     td.append(
                         self.render_action_button(
                             details_url,
-                            "Details",
+                            self.param.details_action_button_text,
                             "fa-id-card",
                             name="grid-details-button",
                         )
@@ -845,7 +904,10 @@ class Grid:
                     edit_url += "/%s?%s" % (row_id, self.referrer)
                     td.append(
                         self.render_action_button(
-                            edit_url, "Edit", "fa-edit", name="grid-edit-button"
+                            edit_url,
+                            self.param.edit_action_button_text,
+                            "fa-edit",
+                            name="grid-edit-button",
                         )
                     )
 
@@ -858,7 +920,7 @@ class Grid:
                     td.append(
                         self.render_action_button(
                             delete_url,
-                            "Delete",
+                            self.param.delete_action_button_text,
                             "fa-trash",
                             additional_classes="confirmation",
                             message="Delete record",
@@ -868,15 +930,18 @@ class Grid:
                     )
                 if self.param.post_action_buttons:
                     for btn in self.param.post_action_buttons:
+                        if btn.onclick:
+                            btn.url = None
                         td.append(
                             self.render_action_button(
                                 btn.url,
                                 btn.text,
                                 btn.icon,
+                                _onclick=btn.onclick,
                                 additional_classes=btn.additional_classes,
                                 message=btn.message,
                                 row_id=row_id if btn.append_id else None,
-                                row=row
+                                row=row,
                             )
                         )
                 tr.append(td)
@@ -926,7 +991,7 @@ class Grid:
             grid_header.append(
                 self.render_action_button(
                     create_url,
-                    "New",
+                    self.param.new_action_button_text,
                     "fa-plus",
                     icon_size="normal",
                     override_classes=self.param.grid_class_style.classes.get(
