@@ -1,17 +1,18 @@
 import json
-import jwt
+import hashlib
 import time
 import uuid
+import base64
 from py4web import request, abort
 from py4web.core import Fixture, Session
 
 
 class URLVerifier(Fixture):
     """This class checks for the validity of URL signatures.
-     Specifically, an object of this class can be passed as argument
-     to action.uses() to check for the validity of signatures, and the
-     sign() method can be used to sign a URL.  If an object of this class
-     is passed to the URL helper, it can be used to sign a URL."""
+    Specifically, an object of this class can be passed as argument
+    to action.uses() to check for the validity of signatures, and the
+    sign() method can be used to sign a URL.  If an object of this class
+    is passed to the URL helper, it can be used to sign a URL."""
 
     def __init__(self, url_signer):
         super().__init__()
@@ -22,19 +23,44 @@ class URLVerifier(Fixture):
     def on_request(self):
         """Checks the request's signature"""
         # extra and remove the signature from the query
-        token = request.query.get("_signature")
-        if token is None:
+        signature = request.query.get("_signature")
+        if signature is None:
             abort(403)
         try:
-            key = self.url_signer.get_url_key(request.fullpath, request.query)
-            jwt.decode(token, key, algorithms=["HS256"])
+            h = self.url_signer.algo(self.url_signer.get_key())
+            signature = request.query["_signature"]
+            sig_content = base64.b85decode(signature.encode("utf-8")).decode("utf-8")
+            sig_dict = json.loads(sig_content)
+            ts = sig_dict["ts"]
+            salt = sig_dict["salt"]
+            sig = sig_dict["sig"]
+            h.update(
+                self.url_signer.get_info_to_sign(
+                    request.fullpath, request.query, ts, salt
+                )
+            )
+            computed_sig = base64.b85encode(h.digest()).decode("utf-8")
+            if sig != computed_sig:
+                abort(403)
             # We remove the signature, not to pollute the request.
             del request.query["_signature"]
+            # Checks the expiration time.
+            if self.url_signer.lifespan is not None:
+                if float(ts) + self.url_signer.lifespan < time.time():
+                    abort(403)
         except:
             abort(403)
 
+    def _decode_ts(self, ts_string):
+        """Decodes the timestamp, removing the salt."""
+        s = base64.b85encode(ts_string.encode("utf-8")).decode("utf-8")
+        return float(s.split(";")[1])
+
 
 class URLSigner(Fixture):
+
+    RESERVED_VARIABLES = ["_ts", "_signature"]
+
     def __init__(
         self,
         session=None,
@@ -42,6 +68,7 @@ class URLSigner(Fixture):
         variables_to_sign=None,
         signing_info=None,
         lifespan=None,
+        algo=None,
     ):
         """
         Signer for URLs.
@@ -55,6 +82,7 @@ class URLSigner(Fixture):
             the user id among the things that should not change by doing:
             signing_info = lambda : str(self.session.get("user", {}).get("id", ""))
         :param lifespan: Lifespan of the signature, in seconds.
+        :param algo: Algorithm to use for signing. Default is hashlib.sha256().
 
         The usage is as follows, typically.
 
@@ -84,8 +112,9 @@ class URLSigner(Fixture):
         self.signing_info = signing_info
         self.lifespan = lifespan
         assert "_signature" not in self.variables_to_sign
+        self.algo = algo or hashlib.sha256
 
-    def _get_key(self):
+    def get_key(self):
         """Gets the signing key, creating it if necessary."""
         if self.session is None:
             key = self.key
@@ -94,30 +123,33 @@ class URLSigner(Fixture):
             if key is None:
                 key = str(uuid.uuid1())
                 self.session["_signature_key"] = key
-        return key
+        return key.encode("utf8")
 
-    def get_url_key(self, url, variables):
-        # The key consists of the key, and of the URL parameters.
-        additional_key = {
-            "url": url,
-            "info": self.signing_info() if self.signing_info is not None else "",
-            "vars": {v: repr(variables.get(v)) for v in self.variables_to_sign},
-        }
-        key = self._get_key() + "." + json.dumps(additional_key)
-        return key
+    def get_info_to_sign(self, url, variables, ts, salt):
+        """Gathers the information to be signed."""
+        # The key consists of the url, and of the URL parameters.
+        return json.dumps(
+            {
+                "url": url,
+                "info": self.signing_info() if self.signing_info is not None else "",
+                "vars": {v: str(variables.get(v)) for v in self.variables_to_sign},
+                "ts": ts,
+                "salt:": salt,
+            }
+        ).encode("utf-8")
 
     def sign(self, url, variables):
         """Signs the URL"""
-        # The payload consists of a timestamp, and of additonal parameters.
-        payload = {"ts": str(time.time())}
-        if self.lifespan is not None:
-            payload["exp"] = time.time() + self.lifespan
-        key = self.get_url_key(url, variables)
-        return jwt.encode(payload, key, algorithm="HS256").decode("utf-8")
-
-    def sign_vars(self, url, variables):
-        """Signs a URL, adding to vars (the variables of the URL) a signature."""
-        variables["_signature"] = self.sign(url, variables)
+        for v in self.RESERVED_VARIABLES:
+            assert v not in variables
+        h = self.algo(self.get_key())
+        ts = "%.3f" % time.time()
+        salt = str(uuid.uuid1())
+        h.update(self.get_info_to_sign(url, variables, ts, salt))
+        sig = base64.b85encode(h.digest()).decode("utf-8")
+        sig_content = json.dumps(dict(ts=ts, salt=salt, sig=sig))
+        signature = base64.b85encode(sig_content.encode("utf-8")).decode("utf-8")
+        variables["_signature"] = signature
 
     def verify(self):
         """returns a fixture that verifies the URL and optionally the query_keys"""
